@@ -7,12 +7,16 @@ Created on Mar 18, 2013
 
 import abc 
 
-__version__ = "0.2"
+__version__ = "0.3"
 __all__ = ['StateMachine','State','Event','__version__']
 
+class Context(object):
+    pass
+
 class StateMachine(object):
     
-    def __init__(self,initialState = None):
+    def __init__(self,initialState = None,context= None):
+        self.context = context if context else Context()
         self.state = None
         self.initialState = initialState
         self.states = {}
@@ -29,10 +33,6 @@ class StateMachine(object):
         self.states[state.name] = state
         
     
-    def transition(self,state):
-        assert isinstance(state,basestring)
-        self.state = self.states[state]
-        self.state.doJob()
     
     def start(self,initialState=None):
         if initialState:
@@ -40,7 +40,18 @@ class StateMachine(object):
         assert len(self.states), 'At least one state must be added to machine.'
         assert self.initialState!=None, "You must specify initialState"
         
-        self.transition(self.initialState)
+        def _transition(state):
+            self.state = self.states[state]
+            self.state.doJob()
+            if hasattr(self.state,'nextState'):
+                return self.state.nextState
+            
+        _nextState = self.initialState
+        while True:
+            assert isinstance(_nextState,basestring)
+            _nextState = _transition(_nextState)
+            if not _nextState:
+                break
 
 class Event(object):
     __metaclass__ = abc.ABCMeta
@@ -81,12 +92,9 @@ class State(object):
         self.raiseEvent("start")
         self.eval()
         self.raiseEvent("finish")
-        #assert hasattr(self,'nextState'), "state: %s" % self.name
-        if hasattr(self,'nextState'): 
-            self.machine.transition(self.nextState)
         
     @property
-    def context(self,state):
+    def context(self):
         return self.machine.context
 
     @abc.abstractmethod
@@ -101,10 +109,11 @@ class State(object):
 # Test cases
 if __name__ == '__main__':
     import time
-    SPEED = 1
+    SPEED = .01
 
     class Starting(State):
         def eval(self):
+            #self.context.i = 0
             print 'Starting'
             time.sleep(SPEED)
         
@@ -119,6 +128,8 @@ if __name__ == '__main__':
         
         def eval(self):
             print 'Listening'
+            print self.context.i
+            self.context.i += 1
             time.sleep(SPEED)
             self.raiseEvent("connect")
 
@@ -148,7 +159,9 @@ if __name__ == '__main__':
             self.transition("Listening")
             
 
-    m = StateMachine(initialState="Starting")
+    ctx = Context()
+    ctx.i = 0
+    m = StateMachine(initialState="Starting",context=ctx)
     m.appendStates(Starting(),Listening(),Negotiation(),Connected())
     m.start()
 
